{#-
  This file was automatically generated - do not edit
-#}
{% extends "base.html" %}
{% block extrahead %}
  <link rel="stylesheet" href="{{ 'overrides/assets/stylesheets/main.d9227bb8.min.css' | url }}">
{% endblock %}
{% block announce %}
  <a href="https://twitter.com/squidfunk">
    For updates follow <strong>@squidfunk</strong> on
    <span class="twemoji twitter">
      {% include ".icons/fontawesome/brands/twitter.svg" %}
    </span>
    <strong>Twitter</strong>
  </a>
{% endblock %}
{% block scripts %}
  {{ super() }}
  <script src="{{ 'overrides/assets/javascripts/bundle.068fb875.min.js' | url }}"></script>
{% endblock %}
