<!--
  Copyright (c) 2016-2022 Martin Donath <martin.donath@squidfunk.com>

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to
  deal in the Software without restriction, including without limitation the
  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
  sell copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
  IN THE SOFTWARE.
-->

<!-- Wrap everything with a macro to reduce file roundtrips (see #2213) -->
{% macro render(nav_item, path, level) %}

  <!-- Determine class according to state -->
  {% set class = "md-nav__item" %}
  {% if nav_item.active %}
    {% set class = class ~ " md-nav__item--active" %}
  {% endif %}

  <!-- Main navigation item with nested items -->
  {% if nav_item.children %}

    <!-- Determine whether to render item as a section -->
    {% if "navigation.sections" in features and level == 1 + (
      "navigation.tabs" in features
    ) %}
      {% set class = class ~ " md-nav__item--section" %}
    {% endif %}

    <!-- Render item with nested items -->
    <li class="{{ class }} md-nav__item--nested">

      <!-- Active checkbox expands items contained within nested section -->
      {% set checked = "checked" if nav_item.active %}
      {% if "navigation.expand" in features and not checked %}
        <input
          class="md-nav__toggle md-toggle"
          data-md-toggle="{{ path }}"
          data-md-state="indeterminate"
          type="checkbox"
          id="{{ path }}"
          checked
        />
      {% else %}
        <input
          class="md-nav__toggle md-toggle"
          data-md-toggle="{{ path }}"
          type="checkbox"
          id="{{ path }}"
          {{ checked }}
        />
      {% endif %}

      <!-- Determine all nested items that are index pages -->
      {% set indexes = [] %}
      {% if "navigation.indexes" in features %}
        {% for nav_item in nav_item.children %}
          {% if nav_item.is_index and not index is defined %}
            {% set _ = indexes.append(nav_item) %}
          {% endif %}
        {% endfor %}
      {% endif %}

      <!-- Render toggle to expand nested items -->
      {% if not indexes %}
        <label class="md-nav__link" for="{{ path }}">
          {{ nav_item.title }}
          <span class="md-nav__icon md-icon"></span>
        </label>

      <!-- Render link to index page + toggle -->
      {% else %}
        {% set index = indexes | first %}
        {% set class = "md-nav__link--active" if index == page %}
        <div class="md-nav__link md-nav__link--index {{ class }}">
          <a href="{{ index.url | url }}">{{ nav_item.title }}</a>

          <!-- Only render toggle if there's at least one more page -->
          {% if nav_item.children | length > 1 %}
            <label for="{{ path }}">
              <span class="md-nav__icon md-icon"></span>
            </label>
          {% endif %}
        </div>
      {% endif %}

      <!-- Render nested navigation -->
      <nav
        class="md-nav"
        aria-label="{{ nav_item.title }}"
        data-md-level="{{ level }}"
      >
        <label class="md-nav__title" for="{{ path }}">
          <span class="md-nav__icon md-icon"></span>
          {{ nav_item.title }}
        </label>
        <ul class="md-nav__list" data-md-scrollfix>

          <!-- Render nested item list -->
          {% for nav_item in nav_item.children %}
            {% if not indexes or nav_item != indexes | first %}
              {{ render(nav_item, path ~ "_" ~ loop.index, level + 1) }}
            {% endif %}
          {% endfor %}
        </ul>
      </nav>
    </li>

  <!-- Currently active page -->
  {% elif nav_item == page %}
    <li class="{{ class }}">
      {% set toc = page.toc %}

      <!-- Active checkbox expands items contained within nested section -->
      <input
        class="md-nav__toggle md-toggle"
        data-md-toggle="toc"
        type="checkbox"
        id="__toc"
      />

      <!-- Hack: see partials/toc.html for more information -->
      {% set first = toc | first %}
      {% if first and first.level == 1 %}
        {% set toc = first.children %}
      {% endif %}

      <!-- Render table of contents, if not empty -->
      {% if toc %}
        <label class="md-nav__link md-nav__link--active" for="__toc">
          {{ nav_item.title }}
          <span class="md-nav__icon md-icon"></span>
        </label>
      {% endif %}
      <a
        href="{{ nav_item.url | url }}"
        class="md-nav__link md-nav__link--active"
      >
        {{ nav_item.title }}
      </a>

      <!-- Show table of contents -->
      {% if toc %}
        {% include "partials/toc.html" %}
      {% endif %}
    </li>

  <!-- Main navigation item -->
  {% else %}
    <li class="{{ class }}">
      <a href="{{ nav_item.url | url }}" class="md-nav__link">
        {{ nav_item.title }}
      </a>
    </li>
  {% endif %}
{% endmacro %}

<!-- Render current and nested navigation items -->
{{ render(nav_item, path, level) }}
