////
/// Copyright (c) 2016-2022 Martin Donath <martin.donath@squidfunk.com>
///
/// Permission is hereby granted, free of charge, to any person obtaining a
/// copy of this software and associated documentation files (the "Software"),
/// to deal in the Software without restriction, including without limitation
/// the rights to use, copy, modify, merge, publish, distribute, sublicense,
/// and/or sell copies of the Software, and to permit persons to whom the
/// Software is furnished to do so, subject to the following conditions:
///
/// The above copyright notice and this permission notice shall be included in
/// all copies or substantial portions of the Software.
///
/// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
/// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
/// FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
/// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
/// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
/// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
/// DEALINGS
////

// ----------------------------------------------------------------------------
// Rules
// ----------------------------------------------------------------------------

// Scoped in typesetted content to match specificity of regular content
.md-typeset {

  // Form button
  .md-button {
    display: inline-block;
    padding: px2em(10px) px2em(32px);
    color: var(--md-primary-fg-color);
    font-weight: 700;
    border: px2rem(2px) solid currentcolor;
    border-radius: px2rem(2px);
    cursor: pointer;
    transition:
      color            125ms,
      background-color 125ms,
      border-color     125ms;

    // Primary button
    &--primary {
      color: var(--md-primary-bg-color);
      background-color: var(--md-primary-fg-color);
      border-color: var(--md-primary-fg-color);
    }

    // Button on focus/hover
    &:is(:focus, :hover) {
      color: var(--md-accent-bg-color);
      background-color: var(--md-accent-fg-color);
      border-color: var(--md-accent-fg-color);
    }
  }

  // Form input
  .md-input {
    height: px2rem(36px);
    padding: 0 px2rem(12px);
    font-size: px2rem(16px);
    border-bottom: px2rem(2px) solid var(--md-default-fg-color--lighter);
    border-start-start-radius: px2rem(2px);
    border-start-end-radius: px2rem(2px);
    box-shadow: var(--md-shadow-z1);
    transition:
      border     250ms,
      box-shadow 250ms;

    // Input on focus/hover
    &:is(:focus, :hover) {
      border-bottom-color: var(--md-accent-fg-color);
      box-shadow: var(--md-shadow-z2);
    }

    // Stretch to full width
    &--stretch {
      width: 100%;
    }
  }
}
