{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "lv",
  "clipboard.copy": "Kopēt starpliktuvē",
  "clipboard.copied": "Kopēts starpliktuvē",
  "edit.link.title": "Rediģēt šo lapu",
  "footer.previous": "Iepriekšējais",
  "footer.next": "Nākamais",
  "footer.title": "Kājene",
  "header.title": "Galvene",
  "meta.comments": "Komentārs",
  "meta.source": "Avots",
  "nav.title": "Navigācija",
  "search.placeholder": "Meklēt",
  "search.reset": "Notīrīt",
  "search.result.initializer": "Notiek meklēšanas inicializācija",
  "search.result.placeholder": "Ierakstiet, lai sāktu meklēšanu",
  "search.result.none": "Nav atbilstošu dokumentu",
  "search.result.one": "1 atbilstošs dokuments",
  "search.result.other": "# atbilstoši dokumenti ",
  "search.result.more.one": "1 šajā lapā",
  "search.result.more.other": "# un vairāk šajā lapā",
  "search.result.term.missing": "Trūkstošs",
  "select.language.title": "Izvēlies valodu",
  "select.version.title": "Izvēlies versiju",
  "skip.link.title": "Pāriet uz saturu",
  "source.link.title": "Doties uz repozitoriju",
  "source.file.date.updated": "Pēdējoreiz atjaunots",
  "source.file.date.created": "Izveidots",
  "tabs.title": "Cilnes",
  "toc.title": "Satura rādītājs",
  "top.title": "Atpakaļ uz augšu"
}[key] }}{% endmacro %}
