////
/// Copyright (c) 2016-2022 Martin Donath <martin.donath@squidfunk.com>
///
/// Permission is hereby granted, free of charge, to any person obtaining a
/// copy of this software and associated documentation files (the "Software"),
/// to deal in the Software without restriction, including without limitation
/// the rights to use, copy, modify, merge, publish, distribute, sublicense,
/// and/or sell copies of the Software, and to permit persons to whom the
/// Software is furnished to do so, subject to the following conditions:
///
/// The above copyright notice and this permission notice shall be included in
/// all copies or substantial portions of the Software.
///
/// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
/// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
/// FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
/// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
/// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
/// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
/// DEALINGS
////

// ----------------------------------------------------------------------------
// Rules
// ----------------------------------------------------------------------------

// Tag list
.md-tags {
  margin-bottom: px2em(12px);
}

// Tag
.md-tag {
  display: inline-block;
  margin-inline-end: 0.5em;
  margin-bottom: 0.5em;
  padding: px2em(4px, 12.8px) px2em(12px, 12.8px);
  font-weight: 700;
  font-size: px2rem(12.8px);
  line-height: 1.6;
  background: var(--md-default-fg-color--lightest);
  border-radius: px2rem(8px);

  // Linked tag
  &[href] {
    color: inherit;
    outline: none;
    -webkit-tap-highlight-color: transparent;
    transition:
      color            125ms,
      background-color 125ms;

    // Linked tag on focus/hover
    &:focus,
    &:hover {
      color: var(--md-accent-bg-color);
      background-color: var(--md-accent-fg-color);
    }
  }

  // Tag inside headline
  [id] > & {
    vertical-align: text-top;
  }
}
