{#-
  This file was automatically generated - do not edit
-#}
{% if page and page.meta and page.meta.hide %}
  {% set hidden = "hidden" if "tags" in page.meta.hide %}
{% endif %}
{% if tags %}
  <nav class="md-tags" {{ hidden }}>
    {% for tag in tags %}
      {% if tag.url %}
        <a href="{{ tag.url | url }}" class="md-tag">
          {{ tag.name }}
        </a>
      {% else %}
        <span class="md-tag">{{ tag.name }}</span>
      {% endif %}
    {% endfor %}
  </nav>
{% endif %}
